@@ -1,5 +1,5 @@
 ;; Foreign functions provided by the win32 system libraries
-;; Sam Steingold 2003-2005
+;; Copyright (C) 2003-2007 Sam Steingold
 
 (defpackage "WIN32"
   (:nicknames "WOE32" "W32")
@@ -9,83 +9,48 @@
            #:defconstant #:defun #:defmacro
            #:def-c-type #:def-c-enum #:def-c-struct #:def-c-var #:def-call-out))
 
-(ffi:default-foreign-language :stdc-stdcall) ; WINAPI means __stdcall
-(eval-when (compile) (setq ffi:*foreign-guard* t))
-
 (in-package "W32")
 
+(default-foreign-language :stdc-stdcall) ; WINAPI means __stdcall
+(eval-when (compile) (setq *foreign-guard* t))
 
 (def-c-type handle c-pointer)
 (def-c-type dword uint32)
 (def-c-type word uint16)
 
 ;; this is not necessary: we are not creating a C file anyway
+;; (and thus cannot use def-c-const instead of defconstant below!)
 ;;(c-lines "#define WINVER 0x0500~%#include <windows.h>~%")
 
-;(defconstant system32 (ext:string-concat (ext:getenv "WINDIR") "\\system32\\"))
-(defconstant advapi32      ; (ext:string-concat system32 "advapi32.dll")
-  "advapi32.dll")
-(defconstant kernel32      ; (ext:string-concat system32 "kernel32.dll")
-  "kernel32.dll")
-(defconstant secur32        ; (ext:string-concat system32 "secur32.dll")
-  "secur32.dll")
-(defconstant shell32        ; (ext:string-concat system32 "shell32.dll")
-  "shell32.dll")
-(defconstant user32          ; (ext:string-concat system32 "user32.dll")
-  "user32.dll")
+;;; ==================================== "kernel32.dll"
+(default-foreign-library "kernel32.dll")
 
-(def-call-out GetCommandLineA (:library kernel32)
-  (:arguments) (:return-type c-string))
-
-(def-call-out GetLastError (:library kernel32)
-  (:arguments) (:return-type dword))
-
-(def-call-out GetCurrentProcess (:library kernel32)
-  (:arguments) (:return-type handle))
-
-(def-call-out GetCurrentThread (:library kernel32)
-  (:arguments) (:return-type handle))
-
-(def-call-out GetCurrentProcessId (:library kernel32)
-  (:arguments) (:return-type dword))
-
-(def-call-out GetProcessVersion (:library kernel32)
-  (:arguments (pid dword)) (:return-type dword))
-
-;;; longhorn
-(def-call-out GetProcessId (:library kernel32)
-  (:arguments (hProcess handle)) (:return-type dword))
-
-(def-call-out GetCurrentThreadId (:library kernel32)
-  (:arguments) (:return-type dword))
-
-;;; longhorn
-(def-call-out GetProcessIdOfThread (:library kernel32)
-  (:arguments (hThread handle)) (:return-type dword))
-
-;;; longhorn
-(def-call-out GetThreadId (:library kernel32)
-  (:arguments (hThread handle)) (:return-type dword))
-
-(def-call-out GetThreadPriority (:library kernel32)
-  (:arguments (hThread handle)) (:return-type int))
-
-(def-call-out GetThreadPriorityBoost (:library kernel32)
-  (:arguments (hThread handle) (no-boost (c-ptr boolean) :out))
-  (:return-type boolean))
-
-(def-call-out GetProcessPriorityBoost (:library kernel32)
-  (:arguments (hProcess handle) (no-boost (c-ptr boolean) :out))
-  (:return-type boolean))
-
-(def-call-out CloseHandle (:library kernel32)
-  (:arguments (handle handle)) (:return-type boolean))
+(def-call-out GetCommandLineA (:return-type c-string) (:arguments))
+(def-call-out GetLastError (:return-type dword) (:arguments))
+(def-call-out GetCurrentProcess (:return-type handle) (:arguments))
+(def-call-out GetCurrentThread (:return-type handle) (:arguments))
+(def-call-out GetCurrentProcessId (:return-type dword) (:arguments))
+(def-call-out GetProcessVersion (:return-type dword) (:arguments (pid dword)))
+(def-call-out GetProcessId (:return-type dword) ; longhorn
+  (:arguments (hProcess handle)))
+(def-call-out GetCurrentThreadId (:arguments) (:return-type dword))
+(def-call-out GetProcessIdOfThread (:return-type dword) ; longhorn
+  (:arguments (hThread handle)))
+(def-call-out GetThreadId (:return-type dword) ; longhorn
+  (:arguments (hThread handle)))
+(def-call-out GetThreadPriority (:return-type int)
+  (:arguments (hThread handle)))
+(def-call-out GetThreadPriorityBoost (:return-type boolean)
+  (:arguments (hThread handle) (no-boost (c-ptr boolean) :out)))
+(def-call-out GetProcessPriorityBoost (:return-type boolean)
+  (:arguments (hProcess handle) (no-boost (c-ptr boolean) :out)))
+(def-call-out CloseHandle (:return-type boolean) (:arguments (handle handle)))
 
 (defmacro with-handle ((handle form) &body forms)
-  (multiple-value-bind (body-rest declarations) (SYSTEM::PARSE-BODY body)
+  (multiple-value-bind (body-rest declarations) (SYSTEM::PARSE-BODY forms)
     `(let ((,handle ,form))
-       (DECLARE (READ-ONLY ,handle) ,@declarations)
-       (unwind-protect (progn ,@forms)
+       (declare (read-only ,handle) ,@declarations)
+       (unwind-protect (progn ,@body-rest)
          (when ,handle (CloseHandle ,handle))))))
 
 ;; (c-lines "#include <winnt.h>~%")
@@ -115,11 +80,140 @@
   (PROCESS_ALL_ACCESS
    #.(cl:logior STANDARD_RIGHTS_REQUIRED SYNCHRONIZE #xFFF)))
 
-(def-call-out OpenProcess (:library kernel32)
+(def-call-out OpenProcess (:return-type handle)
   (:arguments (access-flag dword) ; PROCESS
               (inherit-handle boolean)
-              (pid dword))
-  (:return-type handle))
+              (pid dword)))
+
+(eval-when (compile eval load)
+  (defconstant BUFSIZ 4096)     ; <stdio.h>
+  (defconstant MAX_PATH 260))   ; <windef.h>
+
+(def-call-out GetModuleFileNameA (:return-type dword)
+  (:arguments (application-instance-handle handle)
+              (name (c-ptr (c-array-max character #.MAX_PATH)) :out :alloca)
+              (size dword))) ; always pass MAX_PATH as the second argument
+
+(def-call-out GetModuleHandleA (:return-type handle)
+  (:arguments (name c-string)))
+
+;; (c-lines "#include <winicon.h>~%")
+
+(def-call-out GetConsoleTitleA (:return-type dword)
+  (:arguments (buffer (c-ptr (c-array-max character #.BUFSIZ)) :out :alloca)
+              (size dword)))   ; always pass BUFSIZ as the only argument
+
+(def-call-out SetConsoleTitleA (:return-type boolean)
+  (:arguments (title c-string)))
+
+(def-call-out GetConsoleWindow (:return-type handle) (:arguments)
+  (:documentation "http://msdn.microsoft.com/library/en-us/dllproc/base/getconsolescreenbufferinfo.asp"))
+
+(def-c-struct SMALL_RECT (Left short) (Top short) (Right short) (Bottom short))
+(def-c-struct COORD (X short) (Y short))
+(def-c-struct CONSOLE_SCREEN_BUFFER_INFO
+  (dwSize COORD) (dwCursorPosition COORD)
+  (wAttributes word) (srWindow SMALL_RECT) (dwMaximumWindowSize COORD))
+
+(def-call-out GetConsoleScreenBufferInfo (:return-type boolean)
+  (:documentation "http://msdn.microsoft.com/library/en-us/dllproc/base/getconsolescreenbufferinfo.asp")
+  (:arguments (StdHandle handle)
+              (csbiInfo (c-ptr CONSOLE_SCREEN_BUFFER_INFO) :out :alloca)))
+
+;; system information
+(def-call-out GetSystemDirectoryA (:return-type uint)
+  (:arguments (buffer (c-ptr (c-array-max character #.MAX_PATH)) :out :alloca)
+              (size uint))) ; pass MAX_PATH
+(def-call-out GetWindowsDirectoryA (:return-type uint)
+  (:arguments (buffer (c-ptr (c-array-max character #.MAX_PATH)) :out :alloca)
+              (size uint))) ; pass MAX_PATH
+
+(def-call-out GetCurrentDirectoryA (:return-type dword)
+  (:arguments (size dword) ; pass MAX_PATH
+              (buffer (c-ptr (c-array-max character #.MAX_PATH)) :out :alloca)))
+(def-call-out GetVersion (:return-type dword) (:arguments))
+
+;;; i/o
+(def-call-out GetStdHandle (:return-type handle)
+  (:documentation "http://msdn.microsoft.com/library/en-us/dllproc/base/getstdhandle.asp")
+  (:arguments (nStdHandle dword)))
+(def-call-out SetStdHandle (:return-type boolean)
+  (:documentation "http://msdn.microsoft.com/library/en-us/dllproc/base/setstdhandle.asp")
+  (:arguments (nStdHandle dword) (hHandle handle)))
+
+(defconstant STD_INPUT_HANDLE   ; ((DWORD)-10)
+  (- (ash 1 (bitsizeof 'dword)) 10)
+  "Handle to the standard input device.
+Initially, this is a handle to the console input buffer, CONIN$.")
+(defconstant STD_OUTPUT_HANDLE  ; ((DWORD)-11)
+  (- (ash 1 (bitsizeof 'dword)) 11)
+  "Handle to the standard output device.
+Initially, this is a handle to the active console screen buffer, CONOUT$.")
+(defconstant STD_ERROR_HANDLE   ; ((DWORD)-12)
+  (- (ash 1 (bitsizeof 'dword)) 12)
+  "Handle to the standard error device.
+Initially, this is a handle to the active console screen buffer, CONOUT$.")
+
+(def-call-out ReadFile (:return-type boolean)
+  (:documentation "http://msdn.microsoft.com/library/en-us/fileio/fs/readfile.asp")
+  (:arguments (hFile handle)
+              (lpBuffer c-pointer)
+	      (nNumberOfBytesToRead dword)
+              (lpNumberOfBytesRead (c-ptr dword) :out)
+	      (lpOverlapped c-pointer)))
+
+(def-call-out WriteFile (:return-type boolean)
+  (:documentation "http://msdn.microsoft.com/library/en-us/fileio/fs/writefile.asp")
+  (:arguments (hFile handle)
+              (lpBuffer c-pointer)
+	      (nNumberOfBytesToWrite dword)
+              (lpNumberOfBytesWritten (c-ptr dword) :out)
+	      (lpOverlapped c-pointer)))
+
+(def-call-out ReadConsoleA (:return-type boolean)
+  (:documentation "http://msdn.microsoft.com/library/en-us/dllproc/base/readconsole.asp")
+  (:arguments (hConsoleInput handle)
+              (lpBuffer c-pointer)
+              (nNumberOfCharsToRead dword)
+              (lpNumberOfCharsRead (c-ptr dword) :out)
+              (lpReserved c-pointer)))
+
+(def-call-out WriteConsoleA (:return-type boolean)
+  (:documentation "http://msdn.microsoft.com/library/en-us/dllproc/base/writeconsole.asp")
+  (:arguments (hConsoleOutput handle)
+              (lpBuffer c-pointer)
+              (nNumberOfCharsToWrite dword)
+              (lpNumberOfCharsWritten (c-ptr dword) :out)
+              (lpReserved c-pointer)))
+
+;; computer name
+(eval-when (compile eval load)
+  (defconstant MAX_COMPUTERNAME_LENGTH 16)) ; <winbase.h>
+
+(def-call-out GetComputerNameA (:return-type boolean)
+  (:arguments (buffer (c-ptr (c-array-max character #.MAX_COMPUTERNAME_LENGTH))
+                      :out :alloca)
+              (size (c-ptr dword) :in-out))) ; pass MAX_COMPUTERNAME_LENGTH
+
+(def-c-enum COMPUTER_NAME_FORMAT
+  ComputerNameNetBIOS
+  ComputerNameDnsHostname
+  ComputerNameDnsDomain
+  ComputerNameDnsFullyQualified
+  ComputerNamePhysicalNetBIOS
+  ComputerNamePhysicalDnsHostname
+  ComputerNamePhysicalDnsDomain
+  ComputerNamePhysicalDnsFullyQualified
+  ComputerNameMax)
+
+(def-call-out GetComputerNameExA (:return-type boolean)
+  (:arguments (type-name COMPUTER_NAME_FORMAT)
+              (buffer (c-ptr (c-array-max character #.MAX_COMPUTERNAME_LENGTH))
+                      :out :alloca)
+              (size (c-ptr dword) :in-out))) ; pass MAX_COMPUTERNAME_LENGTH
+
+;;;=================================== "user32.dll"
+(default-foreign-library "user32.dll")
 
 ;;(c-lines "#include <winuser.h>~%")
 (def-c-enum EWX                 ; shutdown operation
@@ -129,16 +223,13 @@
   (EWX_FORCE            #x4)
   (EWX_POWEROFF         #x8)
   (EWX_FORCEIFHUNG     #x10))
-(def-call-out ExitWindowsEx (:library user32)
+(def-call-out ExitWindowsEx (:return-type boolean)
   (:arguments (flags uint)      ; EWX
-              (reserved dword))
-  (:return-type boolean))
+              (reserved dword)))
 
 (def-c-enum GR_OBJECTS GR_GDIOBJECTS GR_USEROBJECTS)
-(def-call-out GetGuiResources (:library user32)
-  (:arguments (process handle)
-              (flags dword))    ; GR_OBJECTS
-  (:return-type dword))
+(def-call-out GetGuiResources (:return-type dword)
+  (:arguments (process handle) (flags dword)))    ; GR_OBJECTS
 
 ;; create an icon
 (def-c-enum image_type
@@ -156,17 +247,16 @@
   (LR_CREATEDIBSECTION 8192)
   (LR_COPYFROMRESOURCE #x4000)
   (LR_SHARED 32768))
-(def-call-out LoadImageA (:library user32)
+(def-call-out LoadImageA (:return-type handle)
   (:documentation "http://msdn.microsoft.com/library/en-us/winui/winui/windowsuserinterface/resources/introductiontoresources/resourcereference/resourcefunctions/loadimage.asp")
   (:arguments (application-instance-handle handle)
               (image-name c-string)
               (type uint) (width int) (height int)
-              (options int))    ; load_options
-  (:return-type handle))
-(def-call-out DestroyIcon (:library user32)
+              (options int)))   ; load_options
+
+(def-call-out DestroyIcon (:return-type boolean)
   (:documentation "http://msdn.microsoft.com/library/en-us/winui/winui/windowsuserinterface/resources/icons/iconreference/iconfunctions/destroyicon.asp")
-  (:arguments (hIcon handle))
-  (:return-type boolean))
+  (:arguments (hIcon handle)))
 #| example:
  (setq icon (win32:LoadImageA nil "d:\\gnu\\clisp\\current\\doc\\clisp.ico"
                               win32:IMAGE_ICON 0 0 win32:LR_LOADFROMFILE))
@@ -223,80 +313,54 @@
 (defconstant IDTRYAGAIN 10 "Try Again button was selected.")
 (defconstant IDYES 6 "Yes button was selected.")
 
-(def-call-out MessageBoxA (:library user32)
+(def-call-out MessageBoxA (:return-type int)
   (:arguments (parent handle) (text c-string) (caption c-string) (type uint))
-  (:documentation "http://msdn.microsoft.com/library/en-us/winui/winui/windowsuserinterface/windowing/dialogboxes/dialogboxreference/dialogboxfunctions/messagebox.asp")
-  (:return-type int))
+  (:documentation "http://msdn.microsoft.com/library/en-us/winui/winui/windowsuserinterface/windowing/dialogboxes/dialogboxreference/dialogboxfunctions/messagebox.asp"))
 #| examples:
  (win32:MessageBoxA nil "welcome to clisp" "message from clisp" win32:MB_OK)
  (win32:MessageBoxA nil "welcome to clisp" "message from clisp"
                     (logior win32:MB_YESNOCANCEL win32:MB_ICONWARNING))
 |#
 
-(eval-when (compile eval load)
-  (defconstant BUFSIZ 4096)     ; <stdio.h>
-  (defconstant MAX_PATH 260))   ; <windef.h>
-
-(def-call-out GetModuleFileNameA (:library kernel32)
-  (:arguments (application-instance-handle handle)
-              (name (c-ptr (c-array-max character #.MAX_PATH)) :out :alloca)
-              (size dword))  ; always pass MAX_PATH as the second argument
-  (:return-type dword))
-
-(def-call-out GetModuleHandleA (:library kernel32)
-  (:arguments (name c-string))
-  (:return-type handle))
-
-;; (c-lines "#include <winicon.h>~%")
-
-(def-call-out GetConsoleTitleA (:library kernel32)
-  (:arguments (buffer (c-ptr (c-array-max character #.BUFSIZ)) :out :alloca)
-              (size dword))  ; always pass BUFSIZ as the only argument
-  (:return-type dword))
-
-(def-call-out SetConsoleTitleA (:library kernel32)
-  (:arguments (title c-string))
-  (:return-type boolean))
-
-(def-call-out GetConsoleWindow (:library kernel32)
-  (:documentation "http://msdn.microsoft.com/library/en-us/dllproc/base/getconsolescreenbufferinfo.asp")
-  (:arguments) (:return-type handle))
-
-(def-c-struct SMALL_RECT (Left short) (Top short) (Right short) (Bottom short))
-(def-c-struct COORD (X short) (Y short))
-(def-c-struct CONSOLE_SCREEN_BUFFER_INFO
-  (dwSize COORD) (dwCursorPosition COORD)
-  (wAttributes word) (srWindow SMALL_RECT) (dwMaximumWindowSize COORD))
-
-(def-call-out GetConsoleScreenBufferInfo (:library kernel32)
-  (:documentation "http://msdn.microsoft.com/library/en-us/dllproc/base/getconsolescreenbufferinfo.asp")
-  (:arguments (StdHandle handle)
-              (csbiInfo (c-ptr CONSOLE_SCREEN_BUFFER_INFO) :out :alloca))
-  (:return-type boolean))
-
-;; system information
-(def-call-out GetSystemDirectoryA (:library kernel32)
-  (:arguments (buffer (c-ptr (c-array-max character #.MAX_PATH)) :out :alloca)
-              (size uint)) ; pass MAX_PATH
-  (:return-type uint))
-(def-call-out GetWindowsDirectoryA (:library kernel32)
-  (:arguments (buffer (c-ptr (c-array-max character #.MAX_PATH)) :out :alloca)
-              (size uint)) ; pass MAX_PATH
-  (:return-type uint))
-(def-call-out GetCurrentDirectoryA (:library kernel32)
-  (:arguments (size dword) ; pass MAX_PATH
-              (buffer (c-ptr (c-array-max character #.MAX_PATH)) :out :alloca))
-  (:return-type dword))
-(def-call-out GetVersion (:library kernel32)
-  (:arguments) (:return-type dword))
+;; http://msdn.microsoft.com/library/en-us/winui/winui/windowsuserinterface/dataexchange/clipboard/clipboardreference/clipboardfunctions/getclipboarddata.asp
+(def-call-out OpenClipboard (:return-type boolean)
+  (:arguments (hWndNewOwner handle)))
+(def-call-out CloseClipboard (:return-type boolean) (:arguments))
+(def-call-out EmptyClipboard (:return-type boolean) (:arguments))
+(def-call-out EnumClipboardFormats (:return-type uint)
+  (:arguments (format uint)))
+(def-call-out GetClipboardData (:return-type handle) (:arguments (format uint)))
+(def-call-out GetClipboardFormatNameA (:return-type int)
+  (:arguments (format uint)
+              (buffer (c-ptr (c-array-max character #.BUFSIZ)) :out :alloca)
+              (size int))) ; always pass BUFSIZ as the second argument
+(def-call-out GetClipboardOwner (:return-type handle) (:arguments))
+(def-call-out GetClipboardSequenceNumber (:return-type dword) (:arguments))
+(def-call-out GetClipboardViewer (:return-type handle) (:arguments))
+(def-call-out GetOpenClipboardWindow (:return-type handle) (:arguments))
+(def-call-out GetPriorityClipboardFormat (:return-type int)
+  (:arguments (formats (c-array-max uint #.BUFSIZ)) (size int)))
+(def-call-out IsClipboardFormatAvailable (:return-type boolean)
+  (:arguments (format uint)))
+(def-call-out RegisterClipboardFormat (:return-type uint)
+  (:arguments (format c-string)))
+(def-call-out RemoveClipboardFormatListener (:return-type boolean)
+  (:arguments (hwnd handle)))
+(def-call-out SetClipboardData (:return-type handle)
+  (:arguments (format uint) (data handle)))
+(def-call-out SetClipboardViewer (:return-type handle)
+  (:arguments (hwnd handle)))
 
+;;;===================================== "advapi32.dll"
+(default-foreign-library "advapi32.dll")
 ;; user name
-(eval-when (compile eval load)
-  (defconstant UNLEN 256)) ; <lmcons.h>
-(def-call-out GetUserNameA (:library advapi32)
+(eval-when (compile eval load) (defconstant UNLEN 256)) ; <lmcons.h>
+(def-call-out GetUserNameA (:return-type boolean)
   (:arguments (buffer (c-ptr (c-array-max character #.UNLEN)) :out :alloca)
-              (size (c-ptr dword) :in-out)) ; pass UNLEN
-  (:return-type boolean))
+              (size (c-ptr dword) :in-out))) ; pass UNLEN
+
+;;;==================================== "secur32.dll"
+(default-foreign-library "secur32.dll")
 
 ;; declared in <secext.h>, include <security.h>
 (def-c-enum EXTENDED_NAME_FORMAT
@@ -309,45 +373,17 @@
   (NameUserPrincipal 8)
   (NameCanonicalEx 9)
   (NameServicePrincipal 10))
-(def-call-out GetUserNameExA (:library secur32)
+(def-call-out GetUserNameExA (:return-type boolean)
   (:arguments (name-format EXTENDED_NAME_FORMAT)
               (buffer (c-ptr (c-array-max character #.UNLEN)) :out :alloca)
-              (size (c-ptr ulong) :in-out)) ; pass UNLEN
-  (:return-type boolean))
-(def-call-out GetComputerObjectNameA (:library secur32)
+              (size (c-ptr ulong) :in-out))) ; pass UNLEN
+(def-call-out GetComputerObjectNameA (:return-type boolean)
   (:arguments (name-format EXTENDED_NAME_FORMAT)
               (buffer (c-ptr (c-array-max character #.UNLEN)) :out :alloca)
-              (size (c-ptr ulong) :in-out)) ; pass UNLEN
-  (:return-type boolean))
-
-;; computer name
-(eval-when (compile eval load)
-  (defconstant MAX_COMPUTERNAME_LENGTH 16)) ; <winbase.h>
-
-(def-call-out GetComputerNameA (:library kernel32)
-  (:arguments (buffer (c-ptr (c-array-max character #.MAX_COMPUTERNAME_LENGTH))
-                      :out :alloca)
-              (size (c-ptr dword) :in-out)) ; pass MAX_COMPUTERNAME_LENGTH
-  (:return-type boolean))
-
-(def-c-enum COMPUTER_NAME_FORMAT
-  ComputerNameNetBIOS
-  ComputerNameDnsHostname
-  ComputerNameDnsDomain
-  ComputerNameDnsFullyQualified
-  ComputerNamePhysicalNetBIOS
-  ComputerNamePhysicalDnsHostname
-  ComputerNamePhysicalDnsDomain
-  ComputerNamePhysicalDnsFullyQualified
-  ComputerNameMax)
-
-(def-call-out GetComputerNameExA (:library kernel32)
-  (:arguments (type-name COMPUTER_NAME_FORMAT)
-              (buffer (c-ptr (c-array-max character #.MAX_COMPUTERNAME_LENGTH))
-                      :out :alloca)
-              (size (c-ptr dword) :in-out)) ; pass MAX_COMPUTERNAME_LENGTH
-  (:return-type boolean))
+              (size (c-ptr ulong) :in-out))) ; pass UNLEN
 
+;;;==================================== "shell32.dll"
+(default-foreign-library "shell32.dll")
 ;; http://msdn.microsoft.com/library/en-us/shellcc/platform/shell/reference/functions/shellexecute.asp
 (def-c-enum SE_ERROR            ; <shellapi.h>
   (SE_ERR_FNF 2)
@@ -377,69 +413,9 @@
   (SW_SHOWDEFAULT 10)
   (SW_FORCEMINIMIZE 11)
   (SW_MAX 11))
-(def-call-out ShellExecuteA (:library shell32)
+(def-call-out ShellExecuteA (:return-type int)
   (:arguments (parent handle) (operation c-string) (file c-string)
-              (parameters c-string) (directory c-string) (show SHOW_COMMAND))
-  (:return-type int))
-
-;;; i/o
-(def-call-out GetStdHandle (:library kernel32)
-  (:documentation "http://msdn.microsoft.com/library/en-us/dllproc/base/getstdhandle.asp")
-  (:arguments (nStdHandle dword)) (:return-type handle))
-(def-call-out SetStdHandle (:library kernel32)
-  (:documentation "http://msdn.microsoft.com/library/en-us/dllproc/base/setstdhandle.asp")
-  (:arguments (nStdHandle dword) (hHandle handle))
-  (:return-type boolean))
-
-(defconstant STD_INPUT_HANDLE   ; ((DWORD)-10)
-  (- (ash 1 (bitsizeof 'dword)) 10)
-  "Handle to the standard input device.
-Initially, this is a handle to the console input buffer, CONIN$.")
-(defconstant STD_OUTPUT_HANDLE  ; ((DWORD)-11)
-  (- (ash 1 (bitsizeof 'dword)) 11)
-  "Handle to the standard output device.
-Initially, this is a handle to the active console screen buffer, CONOUT$.")
-(defconstant STD_ERROR_HANDLE   ; ((DWORD)-12)
-  (- (ash 1 (bitsizeof 'dword)) 12)
-  "Handle to the standard error device.
-Initially, this is a handle to the active console screen buffer, CONOUT$.")
-
-(def-call-out ReadFile (:library kernel32)
-  (:documentation "http://msdn.microsoft.com/library/en-us/fileio/fs/readfile.asp")
-  (:arguments (hFile handle)
-              (lpBuffer c-pointer)
-	      (nNumberOfBytesToRead dword)
-              (lpNumberOfBytesRead (c-ptr dword) :out)
-	      (lpOverlapped c-pointer))
-  (:return-type boolean))
-
-(def-call-out WriteFile (:library kernel32)
-  (:documentation "http://msdn.microsoft.com/library/en-us/fileio/fs/writefile.asp")
-  (:arguments (hFile handle)
-              (lpBuffer c-pointer)
-	      (nNumberOfBytesToWrite dword)
-              (lpNumberOfBytesWritten (c-ptr dword) :out)
-	      (lpOverlapped c-pointer))
-  (:return-type boolean))
-
-(def-call-out ReadConsoleA (:library kernel32)
-  (:documentation "http://msdn.microsoft.com/library/en-us/dllproc/base/readconsole.asp")
-  (:arguments (hConsoleInput handle)
-              (lpBuffer c-pointer)
-              (nNumberOfCharsToRead dword)
-              (lpNumberOfCharsRead (c-ptr dword) :out)
-              (lpReserved c-pointer))
-  (:return-type boolean))
-
-(def-call-out WriteConsoleA (:library kernel32)
-  (:documentation "http://msdn.microsoft.com/library/en-us/dllproc/base/writeconsole.asp")
-  (:arguments (hConsoleOutput handle)
-              (lpBuffer c-pointer)
-              (nNumberOfCharsToWrite dword)
-              (lpNumberOfCharsWritten (c-ptr dword) :out)
-              (lpReserved c-pointer))
-  (:return-type boolean))
-
+              (parameters c-string) (directory c-string) (show SHOW_COMMAND)))
 
 ;; ==========================================================================
 
