@@ -1,7 +1,7 @@
 ;;; Matlab API interface
 ;;; <http://www.mathworks.com/access/helpdesk/help/techdoc/apiref/apiref.shtml>
 ;;;
-;;; Copyright (C) 2004-2005 by Sam Steingold
+;;; Copyright (C) 2004-2006 by Sam Steingold
 ;;; This is Free Software, covered by the GNU GPL (v2)
 ;;; See http://www.gnu.org/copyleft/gpl.html
 
@@ -14,12 +14,14 @@
 
 (in-package "MATLAB")
 
+(setf (documentation (find-package "MATLAB") 'sys::impnotes) "matlab")
+
 ;;; types and constants
 
 
 ;;; foreign function definitions
 
-(ffi:default-foreign-language :stdc)
+(default-foreign-language :stdc)
 
 ;;; --- Engine ---
 (c-lines "#include <engine.h>~%")
@@ -431,4 +433,6 @@
   (:arguments (array_ptr (c-pointer mxArray))))
 
 
+(pushnew :matlab *features*)
 (provide "matlab")
+(pushnew "MATLAB" custom:*system-package-list* :test #'string=)
