@@ -1,4 +1,4 @@
-;;; Copyright (C) 2000-2004 by Sam Steingold
+;;; Copyright (C) 2000-2006 by Sam Steingold
 ;;; This file is a part of CLISP (http://clisp.cons.org), and, as such,
 ;;; is distributed under the GNU GPL (http://www.gnu.org/copyleft/gpl.html)
 
@@ -11,9 +11,14 @@
 
 (defvar *browsers*
   '((:netscape "netscape" "~a")
-    (:netscape-remote "netscape" "-remote" "openURL(~a,new-window)")
+    (:netscape-window "netscape" "-remote" "openURL(~a,new-window)")
+    (:netscape-tab "netscape" "-remote" "openURL(~a,new-tab)")
     (:mozilla "mozilla" "~a")
-    (:mozilla-remote "mozilla" "-remote" "openURL(~a,new-window)")
+    (:mozilla-window "mozilla" "-remote" "openURL(~a,new-window)")
+    (:mozilla-tab "mozilla" "-remote" "openURL(~a,new-tab)")
+    (:firefox "firefox" "~a")
+    (:firefox-window "firefox" "-remote" "openURL(~a,new-window)")
+    (:firefox-tab "firefox" "-remote" "openURL(~a,new-tab)")
     (:konqueror "kfmclient" "openURL" "~a")
     (:lynx "lynx" "~a")
     #+unix (:lynx-xterm "xterm" "-e" "lynx" "~a")
@@ -31,6 +36,11 @@
 (defvar *browser* nil
   "The default browser - a key in `*browsers*' or a list of strings.")
 
+(defun start-message (out caller file size &optional (units "bytes"))
+  (when out
+    (format out "~&;; ~S(~A): ~:D ~A..." caller file size units)
+    (force-output (if (eq out t) *standard-output* out))))
+
 (defun read-from-file (file &key (out *standard-output*) (package "KEYWORD")
                        repeat)
   "Read an object from a file.
@@ -42,10 +52,7 @@ The keyword argument REPEAT specifies how many objects to read:
  if T, read until end of file and return a list of objects read"
   (let ((beg-real (get-internal-real-time)))
     (prog1 (with-open-file (str file :direction :input)
-             (when out
-               (format out "~&;; Reading `~a' [~:d bytes]..."
-                       file (file-length str))
-               (force-output (if (eq out t) *standard-output* out)))
+             (start-message out 'read-from-file file (file-length str))
              (with-standard-io-syntax
                (let ((*package* (find-package package)))
                  (cond ((null repeat) (read str))
@@ -190,9 +197,19 @@ set *HTTP-PROXY*, and return it; otherwise just return *HTTP-PROXY*."
   (cond ((string-equal #1="http://" path :end2 (min len #.(length #1#)))
          (apply #'open-http path options))
         ((string-equal #2="file:/" path :end2 (min len #3=#.(length #2#)))
-         (apply #'open (subseq path (position #\/ path :test-not #'eql
-                                              :start #3#))
-                options))
+         ;; Tomas Zellerin writes in bug 1494059:
+         ;; I think that proper RFC compliant URL of this kind is
+         ;; file://<machine>/<path>, where machine may be the empty string
+         ;; for localhost and path should be an absolute path (including
+         ;; the leading slash on Unix), but browsers usually do not require
+         ;; four slashes in row.
+         (let ((path-beg (position #\/ path :test-not #'eql :start #3#)))
+           ;; we first try stripping all leading slashes to catch things like
+           ;; file:///c:/foo/bar and then resort to keeping one leading #\/
+           (apply #'open (or #+(or win32 cygwin)
+                             (probe-file (subseq path path-beg))
+                             (subseq path (1- path-beg)))
+                  options)))
         (t (open path))))
 
 ;;; the CLHS & IMPNOTES documentation:
