@@ -153,13 +153,42 @@
 
 ;;; 29.4.6. Creating Conditions
 
+(defun find-subclasses-of-type (type class)
+  "Find all subclasses of CLASS that are subtypes of the given TYPE."
+  (if (subtypep class type)
+      (list class)
+      (delete-duplicates
+       (loop :for c :in (clos:class-direct-subclasses class)
+         :nconc (find-subclasses-of-type type c)))))
+
+#+(or) ; not used
+(defun prune-subclasses (classes)
+  "Delete classes that are subclasses of other classes."
+  (do ((tail classes (cdr tail)) this)
+      ((endp tail) (delete nil classes))
+    (setq this (car tail))
+    (when (loop :for c :in classes
+            ;; when THIS is a subclass of C, remove THIS
+            :thereis (and c (not (eq this c)) (clos::subclassp this c)))
+      (setf (car tail) nil))))
+
 ;; MAKE-CONDITION, CLtL2 p. 901
 (defun make-condition (type &rest slot-initializations)
   (unless (subtypep type 'condition)
     (error-of-type 'error
       (TEXT "~S: type ~S is not a subtype of ~S")
       'make-condition type 'condition))
-  (apply #'clos:make-instance type slot-initializations))
+  (let ((class (or (and (symbolp type) (find-class type nil))
+                   ;; not a specific class - find a maximal subclass of
+                   ;; CONDITION that has the given TYPE
+                   (car (last (sort (find-subclasses-of-type
+                                     type (find-class 'condition))
+                                    #'clos::subclassp))))))
+    (if class
+        (apply #'clos:make-instance class slot-initializations)
+        (error-of-type 'error
+          (TEXT "~S: cannot find a ~S class that is a subtype of ~S")
+                'make-condition 'condition type))))
 
 ;; canonicalize a condition argument, CLtL2 p. 888
 
@@ -592,7 +621,7 @@
 
 ;; IGNORE-ERRORS, CLtL2 p. 897
 (defmacro ignore-errors (&body body)
-  (let ((blockname (gensym)))
+  (let ((blockname (gensym "IGNORE-ERRORS-")))
     `(BLOCK ,blockname
        (HANDLER-BIND
          ((ERROR #'(LAMBDA (CONDITION)
@@ -630,10 +659,10 @@
                 :detail varlist
                 (TEXT "~S: too many variables ~S in clause ~S")
                 'handler-case varlist clause)))
-          (push (cons (gensym) clause) extended-clauses))))
+          (push (cons (gensym "HANDLER-") clause) extended-clauses))))
     (setq extended-clauses (nreverse extended-clauses))
-    (let ((blockname (gensym))
-          (tempvar (gensym)))
+    (let ((blockname (gensym "HANDLER-CASE-"))
+          (tempvar (gensym "CONDITION-")))
       `(BLOCK ,blockname
          (LET (,tempvar) ; tempvar is IGNORABLE since it is a gensym
            (TAGBODY
@@ -992,8 +1021,8 @@
                     `(,(gensym) ,name ,test ,interactive ,report ,meaningfulp
                        ,arglist ,@clause))))
             restart-clauses))
-          (blockname (gensym))
-          (arglistvar (gensym))
+          (blockname (gensym "RESTART-"))
+          (arglistvar (gensym "ARGS-"))
           (associate
            ;; Yick.  As a pretty lame way of allowing for an association
            ;; between conditions and restarts, RESTART-CASE has to
@@ -1003,7 +1032,7 @@
                 (case (first form)
                   ((SIGNAL ERROR CERROR WARN ERROR-OF-TYPE) t)
                   (t nil))
-                (gensym))))
+                (gensym "RESTARTS-"))))
       `(BLOCK ,blockname
          (LET (,arglistvar) ; arglistvar is IGNORABLE since it is a gensym
            (TAGBODY
@@ -1097,8 +1126,8 @@
     ;; Here's an example of how we can easily optimize things. There is no
     ;; need to refer to anything in the lexical environment, so we can avoid
     ;; consing a restart at run time.
-    (let ((blockname (gensym))
-          (tag (gensym)))
+    (let ((blockname (gensym "RESTART-"))
+          (tag (gensym "SIMPLE-RESTART-")))
       `(BLOCK ,blockname
          (CATCH ',tag
            (LET ((*ACTIVE-RESTARTS*
@@ -1169,8 +1198,8 @@
 
 ;; CHECK-TYPE, CLtL2 p. 889
 (defmacro check-type (place typespec &optional (string nil) &environment env)
-  (let ((tag1 (gensym))
-        (tag2 (gensym))
+  (let ((tag1 (gensym "CHECK-TYPE-"))
+        (tag2 (gensym "OK-"))
         (var (gensym)))
     `(TAGBODY
        ,tag1
@@ -1200,8 +1229,8 @@
 ;; ASSERT, CLtL2 p. 891
 (defmacro assert (test-form &optional (place-list nil) (datum nil) &rest args
                   &environment env)
-  (let ((tag1 (gensym))
-        (tag2 (gensym)))
+  (let ((tag1 (gensym "ASSERT-"))
+        (tag2 (gensym "OK-")))
     `(TAGBODY
        ,tag1
        (WHEN ,test-form (GO ,tag2))
@@ -1344,14 +1373,23 @@
 
 ;; redefine the function in init.lisp, used by LOAD
 (eval-when (compile) (fmakunbound 'eval-loaded-form)) ; avoid a warning
-(defun eval-loaded-form (obj)
+;; FILE is the *LOAD-TRUENAME* of the file being loaded
+;; we cannot use *LOAD-TRUENAME* because when the error is in a nested LOAD,
+;; we will get the truename of the inner-most file instead of the current one
+(defun eval-loaded-form (obj file)
   (restart-case (eval-loaded-form-low obj)
     (skip
-        :report (lambda (out) (format out (TEXT "skip this form and proceed")))
+        :report (lambda (out)
+                  (format out (TEXT "skip "))
+                  (if (compiled-function-p obj)
+                      (write (sys::closure-name obj) :stream out
+                             :pretty nil :escape nil)
+                      (write obj :stream out :pretty nil :escape nil
+                             :level 2 :length 3)))
         :interactive default-restart-interactive
         () (return-from eval-loaded-form 'skip))
     (stop
-        :report (lambda (out) (format out (TEXT "stop loading file")))
+        :report (lambda (out) (format out (TEXT "stop loading file ~A") file))
         :interactive default-restart-interactive
         () (return-from eval-loaded-form 'stop))))
 
@@ -1389,7 +1427,7 @@
                                       keyclause (list keyclause)))
                               keyclauselist)))
          (simply-error (casename form clauselist errorstring expected-type)
-           (let ((var (gensym)))
+           (let ((var (gensym (string-concat (symbol-name casename) "-KEY-"))))
              `(LET ((,var ,form))
                 (,casename ,var ,@(parenthesize-keys clauselist)
                   ;; if a clause contains an OTHERWISE or T key,
@@ -1397,8 +1435,8 @@
                   (OTHERWISE
                     (ETYPECASE-FAILED ,var ,errorstring ',expected-type))))))
          (retry-loop (casename place clauselist errorstring expected-type env)
-           (let ((g (gensym))
-                 (h (gensym)))
+           (let ((g (gensym (symbol-name casename)))
+                 (h (gensym "RETRY-")))
              `(BLOCK ,g
                 (TAGBODY
                   ,h
@@ -1705,6 +1743,7 @@ non-continuable error or a Ctrl-C interrupt occurs."
 (defun set-global-handler (condition-name handler)
   "Make HANDLER handle CONDITION globally.
 HANDLER should be funcallable (symbol or function).
+If it returns, the next applicable error handler is invoked.
 When HANDLER is nil, remove the global handler for CONDITION.
 Returns the added or removed method(s)."
   (let ((clos::*warn-if-gf-already-called* nil)
@@ -1723,7 +1762,7 @@ Returns the added or removed method(s)."
                       (when old-handler
                         (clos:add-method #'global-handler old-handler)))))
                 t))             ; wants-next-method-p == NIL
-             (list :qualifiers '(progn) :lambda-list '(condition)
+             (list :qualifiers #1='(progn) :lambda-list '(condition)
                    'clos::signature #(1 0 NIL NIL NIL NIL)
                    :specializers (list (find-class condition-name)))))
           ((consp condition-name) ; install all these handlers
@@ -1738,7 +1777,7 @@ Returns the added or removed method(s)."
                  (clos:remove-method #'global-handler method)))
              handlers))
           ((symbolp condition-name) ; remove handler for this condition
-           (let ((method (find-method #'global-handler '(progn)
+           (let ((method (find-method #'global-handler #1#
                                       (list (find-class condition-name)) nil)))
              (when method
                (clos:remove-method #'global-handler method)
